@@ -6,43 +6,69 @@
 
 import sys
 
+try:
+    from httplib import HTTPException
+except ImportError:
+    from http.client import HTTPException
+
+from urllib3.exceptions import MaxRetryError
+
+from selenium.common.exceptions import WebDriverException
+
 from splinter.driver.webdriver.firefox import WebDriver as FirefoxWebDriver
 from splinter.driver.webdriver.remote import WebDriver as RemoteWebDriver
 from splinter.driver.webdriver.chrome import WebDriver as ChromeWebDriver
-from splinter.driver.webdriver.phantomjs import WebDriver as PhantomJSWebDriver
 from splinter.exceptions import DriverNotFoundError
 
 
 _DRIVERS = {
-    'firefox': FirefoxWebDriver,
-    'remote': RemoteWebDriver,
-    'chrome': ChromeWebDriver,
-    'phantomjs': PhantomJSWebDriver,
+    "firefox": FirefoxWebDriver,
+    "remote": RemoteWebDriver,
+    "chrome": ChromeWebDriver,
 }
 
 if sys.version_info[0] <= 2:
     try:
         from splinter.driver.zopetestbrowser import ZopeTestBrowser
-        _DRIVERS['zope.testbrowser'] = ZopeTestBrowser
+
+        _DRIVERS["zope.testbrowser"] = ZopeTestBrowser
     except ImportError:
         pass
 
 try:
     import django  # noqa
     from splinter.driver.djangoclient import DjangoClient
-    _DRIVERS['django'] = DjangoClient
+
+    _DRIVERS["django"] = DjangoClient
 except ImportError:
     pass
 
 try:
     import flask  # noqa
     from splinter.driver.flaskclient import FlaskClient
-    _DRIVERS['flask'] = FlaskClient
+
+    _DRIVERS["flask"] = FlaskClient
 except ImportError:
     pass
 
 
-def Browser(driver_name='firefox', *args, **kwargs):
+def get_driver(driver, retry_count=3, *args, **kwargs):
+    """Try to instantiate the driver.
+
+    Common selenium errors are caught and a retry attempt occurs.
+    This can mitigate issues running on Remote WebDriver.
+
+    """
+    for _ in range(retry_count):
+        try:
+            return driver(*args, **kwargs)
+        except (IOError, HTTPException, WebDriverException, MaxRetryError) as e:
+            pass
+
+    raise e
+
+
+def Browser(driver_name="firefox", retry_count=3, *args, **kwargs):
     """
     Returns a driver instance for the given name.
 
@@ -60,4 +86,5 @@ def Browser(driver_name='firefox', *args, **kwargs):
         driver = _DRIVERS[driver_name]
     except KeyError:
         raise DriverNotFoundError("No driver for %s" % driver_name)
-    return driver(*args, **kwargs)
+
+    return get_driver(driver, *args, **kwargs)
