@@ -5,17 +5,17 @@
 # license that can be found in the LICENSE file.
 
 """
-This module contains the basic API for splinter drivers and elemnts.
+This module contains the basic API for splinter drivers and elements.
 """
 
 from splinter.meta import InheritedDocs
-from splinter.request_handler.request_handler import RequestHandler
 
 
-class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
+class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
     """
     Basic driver API class.
     """
+
     driver_name = None
 
     @property
@@ -23,33 +23,43 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
         """
         Title of current page.
         """
-        raise NotImplementedError("%s doesn't support access to the title." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support access to the title." % self.driver_name
+        )
 
     def __enter__(self):
         """
         Context manager to use the browser safely.
         """
-        raise NotImplementedError("%s doesn't support use by 'with' statement." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support use by 'with' statement." % self.driver_name
+        )
 
     def __exit__(self):
         """
         Context manager to use the browser safely.
         """
-        raise NotImplementedError("%s doesn't support use by 'with' statement." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support use by 'with' statement." % self.driver_name
+        )
 
     @property
     def html(self):
         """
         Source of current page.
         """
-        raise NotImplementedError("%s doesn't support access to the html." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support access to the html." % self.driver_name
+        )
 
     @property
     def url(self):
         """
         URL of current page.
         """
-        raise NotImplementedError("%s doesn't support access to the url." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support access to the url." % self.driver_name
+        )
 
     def visit(self, url):
         """
@@ -65,7 +75,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         If there is no URL to back, this method does nothing.
         """
-        raise NotImplementedError("%s doesn't support moving back in history." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support moving back in history." % self.driver_name
+        )
 
     def forward(self):
         """
@@ -73,13 +85,17 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         If there is no URL to forward, this method does nothing.
         """
-        raise NotImplementedError("%s doesn't support moving forward in history." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support moving forward in history." % self.driver_name
+        )
 
     def reload(self):
         """
         Revisits the current URL
         """
-        raise NotImplementedError("%s doesn't support reloading the page." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support reloading the page." % self.driver_name
+        )
 
     def get_alert(self):
         """
@@ -97,16 +113,18 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
         """
         raise NotImplementedError("%s doesn't support frames." % self.driver_name)
 
-    def execute_script(self, script):
+    def execute_script(self, script, *args):
         """
         Executes a given JavaScript in the browser.
 
         e.g.: ::
             >>> browser.execute_script('document.getElementById("body").innerHTML = "<p>Hello world!</p>"')
         """
-        raise NotImplementedError("%s doesn't support executiong of arbitrary JavaScript." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support execution of arbitrary JavaScript." % self.driver_name
+        )
 
-    def evaluate_script(self, script):
+    def evaluate_script(self, script, *args):
         """
         Similar to :meth:`execute_script <DriverAPI.execute_script>` method.
 
@@ -115,21 +133,27 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
         e.g.: ::
             >>> assert 4 == browser.evaluate_script('2 + 2')
         """
-        raise NotImplementedError("%s doesn't support evaluation of arbitrary JavaScript." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support evaluation of arbitrary JavaScript." % self.driver_name
+        )
 
     def find_by_css(self, css_selector):
         """
-        Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`, using a CSS selector to query the
-        current page content.
+        Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`,
+        using a CSS selector to query the current page content.
         """
-        raise NotImplementedError("%s doesn't support finding elements by css selector." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding elements by css selector." % self.driver_name
+        )
 
     def find_by_xpath(self, xpath):
         """
-        Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`, using a xpath selector to query the
-        current page content.
+        Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`,
+        using a xpath selector to query the current page content.
         """
-        raise NotImplementedError("%s doesn't support finding elements by xpath selector." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding elements by xpath selector." % self.driver_name
+        )
 
     def find_by_name(self, name):
         """
@@ -137,15 +161,20 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`.
         """
-        raise NotImplementedError("%s doesn't support finding elements by name." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding elements by name." % self.driver_name
+        )
 
     def find_by_id(self, id):
         """
         Finds an element in current page by its id.
 
-        Even when only one element is find, this method returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
+        Even when only one element is find, this method returns an instance of
+        :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding elements by id." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding elements by id." % self.driver_name
+        )
 
     def find_by_value(self, value):
         """
@@ -153,7 +182,19 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding elements by value." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding elements by value." % self.driver_name
+        )
+
+    def find_by_text(self, text):
+        """
+        Finds elements in current page by their text.
+
+        Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
+        """
+        raise NotImplementedError(
+            "%s doesn't support finding elements by text." % self.driver_name
+        )
 
     def find_by_tag(self, tag):
         """
@@ -161,7 +202,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding elements by tag." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding elements by tag." % self.driver_name
+        )
 
     def find_link_by_href(self, href):
         """
@@ -169,7 +212,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding links by href." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding links by href." % self.driver_name
+        )
 
     def find_link_by_partial_href(self, partial_href):
         """
@@ -177,7 +222,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding links by partial href." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding links by partial href." % self.driver_name
+        )
 
     def find_link_by_text(self, text):
         """
@@ -185,7 +232,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding links by text." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding links by text." % self.driver_name
+        )
 
     def find_link_by_partial_text(self, partial_text):
         """
@@ -193,7 +242,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding links by partial text." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding links by partial text." % self.driver_name
+        )
 
     def find_option_by_value(self, value):
         """
@@ -201,7 +252,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding options by value." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding options by value." % self.driver_name
+        )
 
     def find_option_by_text(self, text):
         """
@@ -209,7 +262,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
         """
-        raise NotImplementedError("%s doesn't support finding options by text." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support finding options by text." % self.driver_name
+        )
 
     def is_text_present(self, text, wait_time=None):
         """
@@ -217,31 +272,43 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Returns True if finds a match for the ``text`` and False if not.
         """
-        raise NotImplementedError("%s doesn't support checking if some text is present in the html. " % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support checking if some text is present in the html. "
+            % self.driver_name
+        )
 
     def type(self, name, value, slowly=False):
         """
         Types the ``value`` in the field identified by ``name``.
 
         It's useful to test javascript events like keyPress, keyUp, keyDown, etc.
+
+        If ``slowly`` is True, this function returns an iterator which will type one character per iteration.
         """
-        raise NotImplementedError("%s doesn't support typing on fields by name." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support typing on fields by name." % self.driver_name
+        )
 
     def fill(self, name, value):
         """
         Fill the field identified by ``name`` with the content specified by ``value``.
         """
-        raise NotImplementedError("%s doesn't support filling fields by name." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support filling fields by name." % self.driver_name
+        )
 
-    def fill_form(self, field_values):
+    def fill_form(self, field_values, form_id=None, name=None):
         """
         Fill the fields identified by ``name`` with the content specified by ``value`` in a dict.
-        
-        Currently, fill_form supports the following fields: text, password, textarea, checkbox, radio and select.
-        
+
+        Currently, fill_form supports the following fields: text, password, textarea, checkbox,
+        radio and select.
+
         Checkboxes should be specified as a boolean in the dict.
         """
-        raise NotImplementedError("%s doesn't support filling forms with a dict." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support filling forms with a dict." % self.driver_name
+        )
 
     def choose(self, name, value):
         """
@@ -254,7 +321,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         Then you're choosing the female gender.
         """
-        raise NotImplementedError("%s doesn't support choosing options." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support choosing options." % self.driver_name
+        )
 
     def check(self, name):
         """
@@ -268,7 +337,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         To unckech a checkbox, take a look in the :meth:`uncheck <DriverAPI.uncheck>` method.
         """
-        raise NotImplementedError("%s doesn't support checking elements." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support checking elements." % self.driver_name
+        )
 
     def uncheck(self, name):
         """
@@ -282,7 +353,9 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         To check a checkbox, take a look in the :meth:`check <DriverAPI.check>` method.
         """
-        raise NotImplementedError("%s doesn't support unchecking elements." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support unchecking elements." % self.driver_name
+        )
 
     def select(self, name, value):
         """
@@ -293,7 +366,10 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
             >>> browser.select("state", "NY")
         """
-        raise NotImplementedError("%s doesn't support selecting options in 'select' element." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support selecting options in 'select' element."
+            % self.driver_name
+        )
 
     def click_link_by_href(self, href):
         """
@@ -319,6 +395,12 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
         """
         return self.find_link_by_partial_text(partial_text).first.click()
 
+    def click_link_by_id(self, id):
+        """
+        Clicks in a link by id.
+        """
+        return self.find_by_id(id).first.click()
+
     def quit(self):
         """
         Quits the browser, closing its windows (if it has one).
@@ -329,105 +411,179 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
     def is_element_present_by_css(self, css_selector, wait_time=None):
         """
-        Verify if the element is present in the current page by css, and wait the specified time in ``wait_time``.
+        Verify if the element is present in the current page by css, and wait the specified
+        time in ``wait_time``.
 
         Returns True if the element is present and False if is not present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is present by css" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by css"
+            % self.driver_name
+        )
 
     def is_element_not_present_by_css(self, css_selector, wait_time=None):
         """
-        Verify if the element is not present in the current page by css, and wait the specified time in ``wait_time``.
+        Verify if the element is not present in the current page by css, and wait the specified time
+        in ``wait_time``.
 
         Returns True if the element is not present and False if is present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is not present by css" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by css"
+            % self.driver_name
+        )
 
     def is_element_present_by_xpath(self, xpath, wait_time=None):
         """
-        Verify if the element is present in the current page by xpath, and wait the specified time in ``wait_time``.
+        Verify if the element is present in the current page by xpath, and wait the specified
+        time in ``wait_time``.
 
         Returns True if the element is present and False if is not present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is present by xpath" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by xpath"
+            % self.driver_name
+        )
 
     def is_element_not_present_by_xpath(self, xpath, wait_time=None):
         """
-        Verify if the element is not present in the current page by xpath, and wait the specified time in ``wait_time``.
+        Verify if the element is not present in the current page by xpath,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is not present and False if is present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is not present by xpath" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by xpath"
+            % self.driver_name
+        )
 
     def is_element_present_by_tag(self, tag, wait_time=None):
         """
-        Verify if the element is present in the current page by tag, and wait the specified time in ``wait_time``.
+        Verify if the element is present in the current page by tag,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is present and False if is not present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is present by tag" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by tag"
+            % self.driver_name
+        )
 
     def is_element_not_present_by_tag(self, tag, wait_time=None):
         """
-        Verify if the element is not present in the current page by tag, and wait the specified time in ``wait_time``.
+        Verify if the element is not present in the current page by tag,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is not present and False if is present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is not present by tag" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by tag"
+            % self.driver_name
+        )
 
     def is_element_present_by_name(self, name, wait_time=None):
         """
-        Verify if the element is present in the current page by name, and wait the specified time in ``wait_time``.
+        Verify if the element is present in the current page by name,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is present and False if is not present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is present by name" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by name"
+            % self.driver_name
+        )
 
     def is_element_not_present_by_name(self, name, wait_time=None):
         """
-        Verify if the element is not present in the current page by name, and wait the specified time in ``wait_time``.
+        Verify if the element is not present in the current page by name,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is not present and False if is present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is not present by name" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by name"
+            % self.driver_name
+        )
 
     def is_element_present_by_value(self, value, wait_time=None):
         """
-        Verify if the element is present in the current page by value, and wait the specified time in ``wait_time``.
+        Verify if the element is present in the current page by value,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is present and False if is not present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is present by value" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by value"
+            % self.driver_name
+        )
 
     def is_element_not_present_by_value(self, value, wait_time=None):
         """
-        Verify if the element is not present in the current page by value, and wait the specified time in ``wait_time``.
+        Verify if the element is not present in the current page by value,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is not present and False if is present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is not present by value" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by value"
+            % self.driver_name
+        )
 
-    def is_element_present_by_id(self, id, wait_time=None):
+    def is_element_present_by_text(self, text, wait_time=None):
         """
-        Verify if the element is present in the current page by id, and wait the specified time in ``wait_time``.
+        Verify if the element is present in the current page by text,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is present and False if is not present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is present by id" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by text"
+            % self.driver_name
+        )
 
-    def is_element_not_present_by_id(self, id, wait_time=None):
+    def is_element_not_present_by_text(self, text, wait_time=None):
         """
-        Verify if the element is present in the current page by id, and wait the specified time in ``wait_time``.
+        Verify if the element is not present in the current page by text,
+        and wait the specified time in ``wait_time``.
 
         Returns True if the element is not present and False if is present.
         """
-        raise NotImplementedError("%s doesn't support verifying if element is not present by id" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by text"
+            % self.driver_name
+        )
+
+    def is_element_present_by_id(self, id, wait_time=None):
+        """
+        Verify if the element is present in the current page by id,
+        and wait the specified time in ``wait_time``.
+
+        Returns True if the element is present and False if is not present.
+        """
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is present by id"
+            % self.driver_name
+        )
+
+    def is_element_not_present_by_id(self, id, wait_time=None):
+        """
+        Verify if the element is present in the current page by id,
+        and wait the specified time in ``wait_time``.
+
+        Returns True if the element is not present and False if is present.
+        """
+        raise NotImplementedError(
+            "%s doesn't support verifying if element is not present by id"
+            % self.driver_name
+        )
 
     def screenshot(self, name=None, suffix=None):
         """
         Takes a screenshot of the current page and saves it locally.
         """
-        raise NotImplementedError("%s doesn't support taking screenshots." % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support taking screenshots." % self.driver_name
+        )
 
     @property
     def cookies(self):
@@ -436,10 +592,12 @@ class DriverAPI(InheritedDocs('_DriverAPI', (RequestHandler,), {})):
 
         For more details, check the :doc:`cookies manipulation section </cookies>`.
         """
-        raise NotImplementedError("%s doesn't support cookies manipulation" % self.driver_name)
+        raise NotImplementedError(
+            "%s doesn't support cookies manipulation" % self.driver_name
+        )
 
 
-class ElementAPI(InheritedDocs('_ElementAPI', (object,), {})):
+class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
     """
     Basic element API class.
 
@@ -448,7 +606,7 @@ class ElementAPI(InheritedDocs('_ElementAPI', (object,), {})):
     Once you have an instance, you can easily access attributes like a ``dict``:
 
         >>> element = browser.find_by_id("link-logo").first
-        >>> assert element['href'] == 'http://splinter.cobrateam.info'
+        >>> assert element['href'] == 'https://splinter.readthedocs.io'
 
     You can also interact with the instance using the methods and properties listed below.
     """
@@ -532,6 +690,12 @@ class ElementAPI(InheritedDocs('_ElementAPI', (object,), {})):
         """
         raise NotImplementedError
 
+    def clear(self):
+        """
+        Reset the field value.
+        """
+        raise NotImplementedError
+
     def fill(self, value):
         """
         Fill the field with the content specified by ``value``.
@@ -543,6 +707,15 @@ class ElementAPI(InheritedDocs('_ElementAPI', (object,), {})):
         Types the ``value`` in the field.
 
         It's useful to test javascript events like keyPress, keyUp, keyDown, etc.
+
+        If ``slowly`` is True, this function returns an iterator which will type one character per iteration.
+
+        Also supports selenium.
+
+        Example:
+            >>> from selenium.webdriver.common.keys import Keys
+            >>> ElementAPI.type(Keys.RETURN)
+
         """
         raise NotImplementedError
 
@@ -552,7 +725,13 @@ class ElementAPI(InheritedDocs('_ElementAPI', (object,), {})):
 
         Example:
 
-            >>> element..select("NY")
+            >>> element.select("NY")
+        """
+        raise NotImplementedError
+
+    def screenshot(self):
+        """
+        Take screenshot of the element.
         """
         raise NotImplementedError
 
