@@ -7,66 +7,69 @@
 import re
 
 from lxml.cssselect import CSSSelector
-from zope.testbrowser.browser import Browser, ListControl
+from zope.testbrowser.browser import Browser, ListControl, SubmitControl
 from splinter.element_list import ElementList
 from splinter.exceptions import ElementDoesNotExist
 from splinter.driver import DriverAPI, ElementAPI
+from splinter.driver.element_present import ElementPresentMixIn
+from splinter.driver.xpath_utils import _concat_xpath_from_str
 from splinter.cookie_manager import CookieManagerAPI
 
 import mimetypes
 import lxml.html
-import mechanize
 import time
 
 
 class CookieManager(CookieManagerAPI):
-
-    def __init__(self, browser_cookies):
-        self._cookies = browser_cookies
+    def __init__(self, driver):
+        self.driver = driver
 
     def add(self, cookies):
         if isinstance(cookies, list):
             for cookie in cookies:
                 for key, value in cookie.items():
-                    self._cookies[key] = value
+                    self.driver.cookies[key] = value
                 return
         for key, value in cookies.items():
-            self._cookies[key] = value
+            self.driver.cookies[key] = value
 
     def delete(self, *cookies):
         if cookies:
             for cookie in cookies:
                 try:
-                    del self._cookies[cookie]
+                    del self.driver.cookies[cookie]
                 except KeyError:
                     pass
         else:
-            self._cookies.clearAll()
+            self.driver.cookies.clearAll()
 
     def all(self, verbose=False):
         cookies = {}
-        for key, value in self._cookies.items():
+        for key, value in self.driver.cookies.items():
             cookies[key] = value
         return cookies
 
     def __getitem__(self, item):
-        return self._cookies[item]
+        return self.driver.cookies[item]
+
+    def __contains__(self, key):
+        return key in self.driver.cookies
 
     def __eq__(self, other_object):
         if isinstance(other_object, dict):
-            return dict(self._cookies) == other_object
+            return dict(self.driver.cookies) == other_object
+        return False
 
 
-class ZopeTestBrowser(DriverAPI):
+class ZopeTestBrowser(ElementPresentMixIn, DriverAPI):
 
     driver_name = "zope.testbrowser"
 
-    def __init__(self, user_agent=None, wait_time=2):
+    def __init__(self, wait_time=2):
         self.wait_time = wait_time
-        mech_browser = self._get_mech_browser(user_agent)
-        self._browser = Browser(mech_browser=mech_browser)
+        self._browser = Browser()
 
-        self._cookie_manager = CookieManager(self._browser.cookies)
+        self._cookie_manager = CookieManager(self._browser)
         self._last_urls = []
 
     def __enter__(self):
@@ -96,7 +99,7 @@ class ZopeTestBrowser(DriverAPI):
 
     @property
     def htmltree(self):
-        return lxml.html.fromstring(self.html.decode('utf-8'))
+        return lxml.html.fromstring(self.html.decode("utf-8"))
 
     @property
     def title(self):
@@ -114,17 +117,26 @@ class ZopeTestBrowser(DriverAPI):
         html = self.htmltree
         element = html.xpath('//option[@value="%s"]' % value)[0]
         control = self._browser.getControl(element.text)
-        return ElementList([ZopeTestBrowserOptionElement(control, self)], find_by="value", query=value)
+        return ElementList(
+            [ZopeTestBrowserOptionElement(control, self)], find_by="value", query=value
+        )
 
     def find_option_by_text(self, text):
         html = self.htmltree
         element = html.xpath('//option[normalize-space(text())="%s"]' % text)[0]
         control = self._browser.getControl(element.text)
-        return ElementList([ZopeTestBrowserOptionElement(control, self)], find_by="text", query=text)
+        return ElementList(
+            [ZopeTestBrowserOptionElement(control, self)], find_by="text", query=text
+        )
 
     def find_by_css(self, selector):
         xpath = CSSSelector(selector).path
-        return self.find_by_xpath(xpath, original_find="css", original_selector=selector)
+        return self.find_by_xpath(
+            xpath, original_find="css", original_selector=selector
+        )
+
+    def get_control(self, xpath_element):
+        return xpath_element
 
     def find_by_xpath(self, xpath, original_find=None, original_selector=None):
         html = self.htmltree
@@ -134,24 +146,44 @@ class ZopeTestBrowser(DriverAPI):
         for xpath_element in html.xpath(xpath):
             if self._element_is_link(xpath_element):
                 return self._find_links_by_xpath(xpath)
-            elif self._element_is_control(xpath_element):
+            elif self._element_is_control(xpath_element) and xpath_element.name:
                 return self.find_by_name(xpath_element.name)
             else:
-                elements.append(xpath_element)
+                elements.append(self.get_control(xpath_element))
 
         find_by = original_find or "xpath"
         query = original_selector or xpath
 
-        return ElementList([ZopeTestBrowserElement(element, self) for element in elements], find_by=find_by, query=query)
+        return ElementList(
+            [ZopeTestBrowserElement(element, self) for element in elements],
+            find_by=find_by,
+            query=query,
+        )
 
     def find_by_tag(self, tag):
-        return self.find_by_xpath('//%s' % tag, original_find="tag", original_selector=tag)
+        return self.find_by_xpath(
+            "//%s" % tag, original_find="tag", original_selector=tag
+        )
 
     def find_by_value(self, value):
-        return self.find_by_xpath('//*[@value="%s"]' % value, original_find="value", original_selector=value)
+        return self.find_by_xpath(
+            '//*[@value="%s"]' % value, original_find="value", original_selector=value
+        )
+
+    def find_by_text(self, text):
+        xpath_str = _concat_xpath_from_str(text)
+        return self.find_by_xpath(
+            xpath_str,
+            original_find="text",
+            original_selector=text,
+        )
 
     def find_by_id(self, id_value):
-        return self.find_by_xpath('//*[@id="%s"][1]' % id_value, original_find="id", original_selector=id_value)
+        return self.find_by_xpath(
+            '//*[@id="%s"][1]' % id_value,
+            original_find="id",
+            original_selector=id_value,
+        )
 
     def find_by_name(self, name):
         elements = []
@@ -164,7 +196,13 @@ class ZopeTestBrowser(DriverAPI):
                 index += 1
             except LookupError:
                 break
-        return ElementList([ZopeTestBrowserControlElement(element, self) for element in elements], find_by="name", query=name)
+            except NotImplementedError:
+                break
+        return ElementList(
+            [ZopeTestBrowserControlElement(element, self) for element in elements],
+            find_by="name",
+            query=name,
+        )
 
     def find_link_by_text(self, text):
         return self._find_links_by_xpath("//a[text()='%s']" % text)
@@ -176,26 +214,33 @@ class ZopeTestBrowser(DriverAPI):
         return self._find_links_by_xpath("//a[contains(@href, '%s')]" % partial_href)
 
     def find_link_by_partial_text(self, partial_text):
-        return self._find_links_by_xpath("//a[contains(normalize-space(.), '%s')]" % partial_text)
+        return self._find_links_by_xpath(
+            "//a[contains(normalize-space(.), '%s')]" % partial_text
+        )
 
     def fill(self, name, value):
         self.find_by_name(name=name).first._control.value = value
 
-    def fill_form(self, field_values):
+    def fill_form(self, field_values, form_id=None, name=None):
+        form = self._browser
+        if name or form_id:
+            form = self._browser.getForm(name=name, id=form_id)
+
         for name, value in field_values.items():
-            element = self.find_by_name(name)
-            control = element.first._control
-            if control.type == 'checkbox':
+            control = form.getControl(name=name)
+
+            if control.type == "checkbox":
                 if value:
                     control.value = control.options
                 else:
                     control.value = []
-            elif control.type == 'radio':
-                control.value = [option for option in control.options if option == value]
-            elif control.type == 'select':
+            elif control.type == "radio":
+                control.value = [
+                    option for option in control.options if option == value
+                ]
+            elif control.type == "select":
                 control.value = [value]
             else:
-                # text, textarea, password, tel
                 control.value = value
 
     def choose(self, name, value):
@@ -211,7 +256,7 @@ class ZopeTestBrowser(DriverAPI):
         control.value = []
 
     def attach_file(self, name, file_path):
-        filename = file_path.split('/')[-1]
+        filename = file_path.split("/")[-1]
         control = self._browser.getControl(name=name)
         content_type, _ = mimetypes.guess_type(file_path)
         control.add_file(open(file_path), content_type, filename)
@@ -219,7 +264,11 @@ class ZopeTestBrowser(DriverAPI):
     def _find_links_by_xpath(self, xpath):
         html = self.htmltree
         links = html.xpath(xpath)
-        return ElementList([ZopeTestBrowserLinkElement(link, self) for link in links], find_by="xpath", query=xpath)
+        return ElementList(
+            [ZopeTestBrowserLinkElement(link, self) for link in links],
+            find_by="xpath",
+            query=xpath,
+        )
 
     def select(self, name, value):
         self.find_by_name(name).first._control.value = [value]
@@ -235,7 +284,7 @@ class ZopeTestBrowser(DriverAPI):
 
     def _is_text_present(self, text):
         try:
-            body = self.find_by_tag('body').first
+            body = self.find_by_tag("body").first
             return text in body.text
         except ElementDoesNotExist:
             # This exception will be thrown if the body tag isn't present
@@ -253,27 +302,20 @@ class ZopeTestBrowser(DriverAPI):
         return False
 
     def _element_is_link(self, element):
-        return element.tag == 'a'
+        return element.tag == "a"
 
     def _element_is_control(self, element):
-        return hasattr(element, 'type')
-
-    def _get_mech_browser(self, user_agent):
-        mech_browser = mechanize.Browser()
-        if user_agent is not None:
-            mech_browser.addheaders = [("User-agent", user_agent), ]
-        return mech_browser
+        return hasattr(element, "type")
 
     @property
     def cookies(self):
         return self._cookie_manager
 
 
-re_extract_inner_html = re.compile(r'^<[^<>]+>(.*)</[^<>]+>$')
+re_extract_inner_html = re.compile(r"^<[^<>]+>(.*)</[^<>]+>$")
 
 
 class ZopeTestBrowserElement(ElementAPI):
-
     def __init__(self, element, parent):
         self._element = element
         self.parent = parent
@@ -301,8 +343,13 @@ class ZopeTestBrowserElement(ElementAPI):
         elements = self._element.cssselect('[value="%s"]' % value)
         return ElementList([self.__class__(element, self) for element in elements])
 
+    def find_by_text(self, text):
+        # Add a period to the xpath to search only inside the parent.
+        xpath_str = '.{}'.format(_concat_xpath_from_str(text))
+        return self.find_by_xpath(xpath_str)
+
     def find_by_id(self, id):
-        elements = self._element.cssselect('#%s' % id)
+        elements = self._element.cssselect("#%s" % id)
         return ElementList([self.__class__(element, self) for element in elements])
 
     @property
@@ -315,7 +362,7 @@ class ZopeTestBrowserElement(ElementAPI):
 
     @property
     def outer_html(self):
-        return lxml.html.tostring(self._element, encoding='unicode').strip()
+        return lxml.html.tostring(self._element, encoding="unicode").strip()
 
     @property
     def html(self):
@@ -326,7 +373,6 @@ class ZopeTestBrowserElement(ElementAPI):
 
 
 class ZopeTestBrowserLinkElement(ZopeTestBrowserElement):
-
     def __init__(self, element, parent):
         super(ZopeTestBrowserLinkElement, self).__init__(element, parent)
         self._browser = parent._browser
@@ -339,13 +385,15 @@ class ZopeTestBrowserLinkElement(ZopeTestBrowserElement):
 
 
 class ZopeTestBrowserControlElement(ZopeTestBrowserElement):
-
     def __init__(self, control, parent):
         self._control = control
         self.parent = parent
 
     def __getitem__(self, attr):
-        return self._control.mech_control.attrs[attr]
+        try:
+            return getattr(self._control._control, attr)
+        except AttributeError:
+            return self._control._control.attrs[attr]
 
     @property
     def value(self):
@@ -367,18 +415,18 @@ class ZopeTestBrowserControlElement(ZopeTestBrowserElement):
     def select(self, value):
         self._control.value = [value]
 
-class ZopeTestBrowserOptionElement(ZopeTestBrowserElement):
 
+class ZopeTestBrowserOptionElement(ZopeTestBrowserElement):
     def __init__(self, control, parent):
         self._control = control
         self.parent = parent
 
     def __getitem__(self, attr):
-        return self._control.mech_item.attrs[attr]
+        return getattr(self._control, attr)
 
     @property
     def text(self):
-        return self._control.mech_item.get_labels()[0]._text
+        return self._control.labels[0]
 
     @property
     def value(self):
@@ -386,4 +434,4 @@ class ZopeTestBrowserOptionElement(ZopeTestBrowserElement):
 
     @property
     def selected(self):
-        return self._control.mech_item._selected
+        return self._control.selected
