@@ -7,7 +7,7 @@
 from splinter.exceptions import ElementDoesNotExist
 
 
-class ElementList(list):
+class ElementList(object):
     """
     List of elements. Each member of the list is (usually) an instance
     of :class:`ElementAPI <splinter.driver.ElementAPI>`.
@@ -28,20 +28,24 @@ class ElementList(list):
         """
         Creates the list.
         """
-        self.extend(list)
+        self._container = []
+        self._container.extend(list)
+
         self.driver = driver
         self.find_by = find_by
         self.query = query
 
     def __getitem__(self, index):
-        if not isinstance(index, int):
+        if not isinstance(index, int) and not isinstance(index, slice):
             return self.first[index]
         try:
-            return super(ElementList, self).__getitem__(index)
+            return self._container[index]
         except IndexError:
             raise ElementDoesNotExist(
                 u'no elements could be found with {0} "{1}"'.format(
-                    self.find_by, self.query))
+                    self.find_by, self.query
+                )
+            )
 
     @property
     def first(self):
@@ -70,6 +74,20 @@ class ElementList(list):
     def __getattr__(self, name):
         try:
             return getattr(self.first, name)
-        except (ElementDoesNotExist, AttributeError):
-            raise AttributeError(u"'{0}' object has no attribute '{1}'".format(
-                self.__class__.__name__, name))
+        except AttributeError:
+            try:
+                return getattr(self._container, name)
+            except AttributeError:
+                raise AttributeError(
+                    u"'{0}' object has no attribute '{1}'".format(
+                        self.__class__.__name__, name
+                    )
+                )
+
+    def __iter__(self):
+        for item in self._container:
+            yield item
+
+    def __len__(self):
+        """__len__ checks the internal container."""
+        return len(self._container)
