@@ -6,10 +6,10 @@
 
 
 class InheritedDocs(type):
-
     def __new__(mcs, class_name, bases, dict):
-        items_to_patch = [(k, v) for k, v in dict.items()
-                          if not k.startswith('__') and not v.__doc__]
+        items_to_patch = [
+            (k, v) for k, v in dict.items() if not k.startswith("__") and not v.__doc__
+        ]
         for name, obj in items_to_patch:
             doc = None
             for base in bases:
@@ -17,7 +17,7 @@ class InheritedDocs(type):
                     doc = getattr(base, name).__doc__
 
                     if doc:
-                        if type(obj) == type(property()) and not obj.fset:
+                        if isinstance(obj, property) and not obj.fset:
                             obj.fget.__doc__ = doc
                             dict[name] = property(fget=obj.fget)
                         else:
